const { IsoDateString } = require("../../Utils/Date/IsoDateString")
const { IsoDateString2UTC } = require("../../Utils/Date/UTC")
const { Enum } = require("../../Enum/Enum")

function IsNumberInRange(value, min, max) {
  value = parseInt(value)
  if(isNaN(value)) return false
  if(value < min) return false
  if(value > max) return false
  return true
}

function DaysInFeburary(year)
{
  year = parseInt(year)
  if(!year) return 28
  var d = new Date(year, 1, 29)
  var month = d.getMonth()
  return month === 2 ? 28 : 29
}

function DaysInMonth(year, month) {
  month = parseInt(month)
  year = parseInt(year)
  switch (month) {
    case 1: case 3: case 5: case 7: case 8: case 10: case 12:
      return 31
    case 4: case 6: case 9: case 11:
      return 30
    case 2:
      return daysInFeburary(year)
    default:
      return 31
  }
}

function MilitaryHour(hour, amPm) {
  hour = parseInt(hour)
  if(isNaN(hour)) { return undefined }
  if(hour===0 || hour === 12) {
    if(amPm === Enum.AmPm.am.value) {
      return 0
    } else if(amPm === Enum.AmPm.pm.value) {
      return 12
    }
  } else if(hour > 0 && hour < 12) {
    if(amPm === Enum.AmPm.am.value) {
      return hour
    } else if(amPm === Enum.AmPm.pm.value) {
      return hour + 12
    }
  }
  return undefined
}

function IsTooEarly(hour) {
  hour = parseInt(hour)
  if(isNaN(hour)) return false

  if(hour < 6 || hour===24) {
    return true
  }
  return false
}


function IsBeginEnd(beginHour, beginMinute, endHour, endMinute) {
  if(beginHour > endHour) return false
  if(beginHour < endHour) return true
  return beginMinute < endMinute
}

function IsTooLong(beginHour, beginMinute, endHour, endMinute, hours) {
  beginHour = parseInt(beginHour)
  beginMinute = parseInt(beginMinute)
  endHour = parseInt(endHour)
  endMinute = parseInt(endMinute)
  hours = parseInt(hours)
  return endHour*60 + endMinute > beginHour*60 + beginMinute + hours*60
}

function IsFuture(year, month, date, hour, minute, timeZoneHourOffset) {
  const iso = IsoDateString(year, month, date, hour, minute, timeZoneHourOffset)
  const utc = IsoDateString2UTC(iso)
  const d = new Date()
  const todayUTC =
  Date.UTC(
    d.getUTCFullYear(),
    d.getUTCMonth(),
    d.getUTCDate(),
    d.getUTCHours(),
    d.getUTCMinutes()
  )
  return utc > todayUTC
}

function IsTooFuture(year, month, date, hour,  minute, timeZoneHourOffset, days) {
  const iso = IsoDateString(year, month, date, hour, minute, timeZoneHourOffset)
  const utc = IsoDateString2UTC(iso)

  const d = new Date()
  const futureUTC =
  Date.UTC(
    d.getUTCFullYear(),
    d.getUTCMonth(),
    d.getUTCDate()+days,
    d.getUTCHours(),
    d.getUTCMinutes()
  )
  return utc > futureUTC
}

module.exports = {
  DaysInMonth,
  MilitaryHour,
  IsTooEarly,
  IsNumberInRange,
  IsBeginEnd,
  IsTooLong,
  IsFuture,
  IsTooFuture
}